@@ -1,6 +1,6 @@
 import itertools
-import logging
 import operator
+import logging
 
 import numpy as np
 from astropy.modeling import fitting, Model, models
@@ -20,6 +20,9 @@ from ..manipulation.utils import excise_regions
 __all__ = ['find_lines_threshold', 'find_lines_derivative', 'fit_lines',
            'estimate_line_parameters']
 
+log = logging.getLogger(__name__)
+
+
 # Define the initial estimators. This are the default methods to use to
 # estimate astropy model parameters. This is based on only a small subset of
 # the astropy models but it was determined that this is a decent start as most
@@ -128,7 +131,9 @@ def find_lines_threshold(spectrum, noise_factor=1):
 
     # Threshold based on noise estimate and factor.
     uncertainty = spectrum.uncertainty
-    inds = np.where(np.abs(spectrum.flux) > (noise_factor*uncertainty.array) *
+    uncert_val = 0 if uncertainty is None else uncertainty.array
+
+    inds = np.where(np.abs(spectrum.flux) > (noise_factor * uncert_val) *
                     spectrum.flux.unit)[0]
     pos_inds = inds[spectrum.flux.value[inds] > 0]
     line_inds_grouped = _consecutive(pos_inds, stepsize=1)
@@ -152,23 +157,7 @@ def find_lines_threshold(spectrum, noise_factor=1):
     else:
         absorption_inds = []
 
-    #
-    # Create the QTable to return the lines
-    #
-
-    qtable = QTable()
-    qtable['line_center'] = list(
-        itertools.chain(
-            *[spectrum.spectral_axis.value[emission_inds],
-              spectrum.spectral_axis.value[absorption_inds]]
-        )) * spectrum.spectral_axis.unit
-    qtable['line_type'] = ['emission'] * len(emission_inds) + \
-                          ['absorption'] * len(absorption_inds)
-    qtable['line_center_index'] = list(
-        itertools.chain(
-            *[emission_inds, absorption_inds]))
-
-    return qtable
+    return _generate_line_list_table(spectrum, emission_inds, absorption_inds)
 
 
 @warn_continuum_below_threshold(threshold=0.01)
@@ -250,10 +239,10 @@ def find_lines_derivative(spectrum, flux_threshold=None):
     else:
         absorption_inds = []
 
-    #
-    # Create the QTable to return the lines
-    #
+    return _generate_line_list_table(spectrum, emission_inds, absorption_inds)
 
+
+def _generate_line_list_table(spectrum, emission_inds, absorption_inds):
     qtable = QTable()
     qtable['line_center'] = list(
         itertools.chain(
@@ -397,7 +386,7 @@ def _fit_lines(spectrum, model, fitter=fitting.LevMarLSQFitter(),
             if uncerts is not None:
                 weights = uncerts.array ** -1
             else:
-                logging.warning("Uncertainty values are not defined, but are "
+                log.warning("Uncertainty values are not defined, but are "
                                 "trying to be used in model fitting.")
         else:
             raise ValueError("Unrecognized value `%s` in keyword argument.",
