@@ -1,8 +1,14 @@
+import os
+import glob
+import logging
+from astropy.nddata.nduncertainty import VarianceUncertainty
+
 import astropy.units as u
 from astropy.units import Quantity
 from astropy.table import Table
 from astropy.io import fits
-from astropy.nddata import StdDevUncertainty
+from astropy.nddata import StdDevUncertainty, InverseVariance
+import logging
 import numpy as np
 import asdf
 from gwcs.wcstools import grid_from_bounding_box
@@ -12,25 +18,76 @@ from ..registers import data_loader
 from ..parsing_utils import read_fileobj_or_hdulist
 
 
-__all__ = ["jwst_x1d_single_loader", "jwst_x1d_multi_loader"]
+__all__ = ["jwst_x1d_single_loader", "jwst_x1d_multi_loader", "jwst_x1d_miri_mrs_loader"]
+
+log = logging.getLogger(__name__)
+
+
+def identify_jwst_miri_mrs(origin, *args, **kwargs):
+    """
+    Check whether the given set of files is a JWST MIRI MRS spectral data product.
+    """
+    input = args[2]
+
+    # if string, it can be either a directory or a glob pattern (this last
+    # one not implemented yet due to astropy choking when passed an invalid
+    # file path string).
+    if isinstance(input, str):
+        if os.path.isdir(input):
+            return True
+
+        if len(glob.glob(input)) > 0:
+            return True
+
+    # or it can be either a list of file names, or a list of file objects
+    elif isinstance(input, (list, tuple)) and len(input) > 0:
+        return True
+
+    return False
+
+
+def _identify_spec1d_fits(origin, extname, *args, **kwargs):
+    """ Generic spec 1d identifier function """
+    is_jwst = _identify_jwst_fits(*args)
+    with read_fileobj_or_hdulist(*args, memmap=False, **kwargs) as hdulist:
+        return (is_jwst and extname in hdulist and (extname, 2) not in hdulist)
+
+
+def _identify_spec1d_multi_fits(origin, extname, *args, **kwargs):
+    """
+    Check whether the given file is a JWST c1d/x1d spectral data product with many slits.
+    """
+    is_jwst = _identify_jwst_fits(*args)
+    with read_fileobj_or_hdulist(*args, memmap=False, **kwargs) as hdulist:
+        return is_jwst and (extname, 2) in hdulist
+
+
+def identify_jwst_c1d_fits(origin, *args, **kwargs):
+    """
+    Check whether the given file is a JWST c1d spectral data product.
+    """
+    return _identify_spec1d_fits(origin, 'COMBINE1D', *args, **kwargs)
+
+
+def identify_jwst_c1d_multi_fits(origin, *args, **kwargs):
+    """
+    Check whether the given file is a JWST c1d spectral data product with many slits.
+    """
+    return _identify_spec1d_multi_fits(origin, 'COMBINE1D', *args, **kwargs)
 
 
 def identify_jwst_x1d_fits(origin, *args, **kwargs):
     """
     Check whether the given file is a JWST x1d spectral data product.
     """
-    is_jwst = _identify_jwst_fits(*args)
-    with read_fileobj_or_hdulist(*args, memmap=False, **kwargs) as hdulist:
-        return (is_jwst and 'EXTRACT1D' in hdulist and ('EXTRACT1D', 2) not in hdulist)
+    return _identify_spec1d_fits(origin, 'EXTRACT1D', *args, **kwargs)
 
 
 def identify_jwst_x1d_multi_fits(origin, *args, **kwargs):
     """
     Check whether the given file is a JWST x1d spectral data product with many slits.
     """
-    is_jwst = _identify_jwst_fits(*args)
-    with read_fileobj_or_hdulist(*args, memmap=False, **kwargs) as hdulist:
-        return is_jwst and ('EXTRACT1D', 2) in hdulist
+    return _identify_spec1d_multi_fits(origin, 'EXTRACT1D', *args, **kwargs)
 
 
 def identify_jwst_s2d_fits(origin, *args, **kwargs):
@@ -75,8 +132,58 @@ def _identify_jwst_fits(*args):
         return False
 
 
-@data_loader("JWST x1d", identifier=identify_jwst_x1d_fits, dtype=Spectrum1D,
-             extensions=['fits'])
+@data_loader(
+    "JWST c1d", identifier=identify_jwst_c1d_fits, dtype=Spectrum1D,
+    extensions=['fits'], priority=10,
+)
+def jwst_c1d_single_loader(file_obj, **kwargs):
+    """
+    Loader for JWST c1d 1-D spectral data in FITS format
+
+    Parameters
+    ----------
+    filename : str
+        The path to the FITS file
+
+    Returns
+    -------
+    Spectrum1D
+        The spectrum contained in the file.
+    """
+    spectrum_list = _jwst_spec1d_loader(file_obj, extname='COMBINE1D', **kwargs)
+    if len(spectrum_list) == 1:
+        return spectrum_list[0]
+    else:
+        raise RuntimeError(f"Input data has {len(spectrum_list)} spectra. "
+                           "Use SpectrumList.read() instead.")
+
+
+@data_loader(
+    "JWST c1d multi", identifier=identify_jwst_c1d_multi_fits,
+    dtype=SpectrumList, extensions=['fits'], priority=10,
+)
+def jwst_c1d_multi_loader(file_obj, **kwargs):
+    """
+    Loader for JWST x1d 1-D spectral data in FITS format
+
+    Parameters
+    ----------
+    file_obj: str, file-like, or HDUList
+          FITS file name, object (provided from name by Astropy I/O Registry),
+          or HDUList (as resulting from astropy.io.fits.open()).
+
+    Returns
+    -------
+    SpectrumList
+        A list of the spectra that are contained in the file.
+    """
+    return _jwst_spec1d_loader(file_obj, extname='COMBINE1D', **kwargs)
+
+
+@data_loader(
+    "JWST x1d", identifier=identify_jwst_x1d_fits, dtype=Spectrum1D,
+    extensions=['fits'], priority=10
+)
 def jwst_x1d_single_loader(file_obj, **kwargs):
     """
     Loader for JWST x1d 1-D spectral data in FITS format
@@ -91,7 +198,7 @@ def jwst_x1d_single_loader(file_obj, **kwargs):
     Spectrum1D
         The spectrum contained in the file.
     """
-    spectrum_list = _jwst_x1d_loader(file_obj, **kwargs)
+    spectrum_list = _jwst_spec1d_loader(file_obj, extname='EXTRACT1D', **kwargs)
     if len(spectrum_list) == 1:
         return spectrum_list[0]
     else:
@@ -99,8 +206,10 @@ def jwst_x1d_single_loader(file_obj, **kwargs):
                            "Use SpectrumList.read() instead.")
 
 
-@data_loader("JWST x1d multi", identifier=identify_jwst_x1d_multi_fits,
-             dtype=SpectrumList, extensions=['fits'])
+@data_loader(
+    "JWST x1d multi", identifier=identify_jwst_x1d_multi_fits,
+    dtype=SpectrumList, extensions=['fits'], priority=10,
+)
 def jwst_x1d_multi_loader(file_obj, **kwargs):
     """
     Loader for JWST x1d 1-D spectral data in FITS format
@@ -116,10 +225,72 @@ def jwst_x1d_multi_loader(file_obj, **kwargs):
     SpectrumList
         A list of the spectra that are contained in the file.
     """
-    return _jwst_x1d_loader(file_obj, **kwargs)
+    return _jwst_spec1d_loader(file_obj, extname='EXTRACT1D', **kwargs)
 
 
-def _jwst_x1d_loader(file_obj, **kwargs):
+@data_loader(
+    "JWST x1d MIRI MRS", identifier=identify_jwst_miri_mrs, dtype=SpectrumList,
+    extensions=['*'], priority=10,
+)
+def jwst_x1d_miri_mrs_loader(input, missing="raise", **kwargs):
+    """
+    Loader for JWST x1d MIRI MRS spectral data in FITS format.
+
+    A single data set consists of a bunch of _x1d files corresponding to
+    a variety of wavelength bands. This reader reads them one by one and packs
+    the result into a SpectrumList instance.
+
+    Parameters
+    ----------
+    input : list of str or file-like
+        List of FITS file names, or objects (provided from name by
+        Astropy I/O Registry). Alternatively, a directory path on
+        which glob.glob runs with pattern an implicit pattern "_x1d.fits",
+        or a directory path with a glob pattern already set.
+    missing : {'warn', 'silent'}
+        Allows the user to continue loading if one file is missing by setting
+        the value to "warn" or "silent". In the first case a warning will be issued
+        to the user, in the latter the file will silently be skipped. Any other
+        value will result in a FileNotFoundError if any files in the list are missing.
+
+    Returns
+    -------
+    SpectrumList
+        A list of the spectra that are contained in all the files.
+    """
+    # If input is a list, go read each file. If directory, glob-expand
+    # list of file names.
+    if not isinstance(input, (list, tuple)):
+        if os.path.isdir(input):
+            file_list = glob.glob(os.path.join(input, "*_x1d.fits"), recursive=True)
+        else:
+            file_list = glob.glob(input, recursive=True)
+    else:
+        file_list = input
+
+    spectra = []
+    for file_obj in file_list:
+        try:
+            sp =  _jwst_spec1d_loader(file_obj, **kwargs)
+        except FileNotFoundError as e:
+            if missing.lower() == "warn":
+                log.warning(f'Failed to load {file_obj}: {repr(e)}')
+                continue
+            elif missing.lower() == "silent":
+                continue
+            else:
+                raise FileNotFoundError(f"Failed to load {file_obj}: {repr(e)}. "
+                                        "To suppress this error, set argument missing='warn'")
+
+        # note that the method above returns a single Spectrum1D instance
+        # packaged in a SpectrumList wrapper. We remove the wrapper so as
+        # to avoid ending up with a depth-2 SpectrumList.
+        spectra.append(sp[0])
+
+    return SpectrumList(spectra)
+
+
+def _jwst_spec1d_loader(file_obj, extname='EXTRACT1D', **kwargs):
     """Implementation of loader for JWST x1d 1-D spectral data in FITS format
 
     Parameters
@@ -127,6 +298,8 @@ def _jwst_x1d_loader(file_obj, **kwargs):
     file_obj: str, file-like, or HDUList
           FITS file name, object (provided from name by Astropy I/O Registry),
           or HDUList (as resulting from astropy.io.fits.open()).
+    extname: str
+        The name of the science extension. Either "COMBINE1D" or "EXTRACT1D".  By default "EXTRACT1D".
 
     Returns
     -------
@@ -134,6 +307,9 @@ def _jwst_x1d_loader(file_obj, **kwargs):
         A list of the spectra that are contained in the file.
     """
 
+    if extname not in ['COMBINE1D', 'EXTRACT1D']:
+        raise ValueError('Incorrect extname given for 1d spectral data.')
+
     spectra = []
 
     with read_fileobj_or_hdulist(file_obj, memmap=False, **kwargs) as hdulist:
@@ -141,8 +317,8 @@ def _jwst_x1d_loader(file_obj, **kwargs):
         primary_header = hdulist["PRIMARY"].header
 
         for hdu in hdulist:
-            # Read only the BinaryTableHDUs named EXTRACT1D
-            if hdu.name != 'EXTRACT1D':
+            # Read only the BinaryTableHDUs named COMBINE1D/EXTRACT1D and SCI
+            if hdu.name != extname:
                 continue
 
             data = Table.read(hdu)
@@ -150,20 +326,36 @@ def _jwst_x1d_loader(file_obj, **kwargs):
             wavelength = Quantity(data["WAVELENGTH"])
 
             # Determine if FLUX or SURF_BRIGHT column should be returned
-            # based on whether it is point or extended source
-            srctype = hdu.header.get("srctype")
-            # SRCTYPE is in primary header because there is only one spectrum
-            if srctype is None:
-                srctype = hdulist["PRIMARY"].header.get("srctype")
+            # based on whether it is point or extended source.
+            #
+            # SRCTYPE used to be in primary header, but was moved around. As
+            # per most recent pipeline definition, it should be in the
+            # EXTRACT1D extension.
+            #
+            # SRCTYPE should either be POINT or EXTENDED.  In some cases, it is UNKNOWN
+            # or missing.  If that's the case, default to using POINT as the SRCTYPE.
+            # Error out only when SRCTYPE is a bad value.
+            srctype = None
+            if "srctype" in hdu.header:
+                srctype = hdu.header.get("srctype", None)
+
+            # checking if SRCTPYE is missing or UNKNOWN
+            if not srctype or srctype == 'UNKNOWN':
+                log.warning('SRCTYPE is missing or UNKNOWN in JWST x1d loader. '
+                            'Defaulting to srctype="POINT".')
+                srctype = 'POINT'
 
             if srctype == "POINT":
                 flux = Quantity(data["FLUX"])
-                uncertainty = StdDevUncertainty(data["ERROR"])
-
+                if 'ERROR' in data.colnames:
+                    uncertainty = StdDevUncertainty(data["ERROR"])
+                elif 'FLUX_ERROR' in data.colnames:
+                    uncertainty = StdDevUncertainty(data["FLUX_ERROR"])
+                else:
+                    uncertainty = None
             elif srctype == "EXTENDED":
                 flux = Quantity(data["SURF_BRIGHT"])
                 uncertainty = StdDevUncertainty(hdu.data["SB_ERROR"])
-
             else:
                 raise RuntimeError(f"Keyword SRCTYPE is {srctype}.  It should "
                                    "be 'POINT' or 'EXTENDED'. Can't decide between `flux` and "
@@ -182,8 +374,10 @@ def _jwst_x1d_loader(file_obj, **kwargs):
     return SpectrumList(spectra)
 
 
-@data_loader("JWST s2d", identifier=identify_jwst_s2d_fits, dtype=Spectrum1D,
-             extensions=['fits'])
+@data_loader(
+    "JWST s2d", identifier=identify_jwst_s2d_fits, dtype=Spectrum1D,
+    extensions=['fits'], priority=10,
+)
 def jwst_s2d_single_loader(filename, **kwargs):
     """
     Loader for JWST s2d 2D rectified spectral data in FITS format.
@@ -208,8 +402,10 @@ def jwst_s2d_single_loader(filename, **kwargs):
         raise RuntimeError(f"Input data has {len(spectrum_list)} spectra.")
 
 
-@data_loader("JWST s2d multi", identifier=identify_jwst_s2d_multi_fits, dtype=SpectrumList,
-             extensions=['fits'])
+@data_loader(
+    "JWST s2d multi", identifier=identify_jwst_s2d_multi_fits,
+    dtype=SpectrumList, extensions=['fits'], priority=10,
+)
 def jwst_s2d_multi_loader(filename, **kwargs):
     """
     Loader for JWST s2d 2D rectified spectral data in FITS format.
@@ -320,8 +516,10 @@ def _jwst_s2d_loader(filename, **kwargs):
     return SpectrumList(spectra)
 
 
-@data_loader("JWST s3d", identifier=identify_jwst_s3d_fits, dtype=Spectrum1D,
-             extensions=['fits'])
+@data_loader(
+    "JWST s3d", identifier=identify_jwst_s3d_fits, dtype=Spectrum1D,
+    extensions=['fits'], priority=10,
+)
 def jwst_s3d_single_loader(filename, **kwargs):
     """
     Loader for JWST s3d 3D rectified spectral data in FITS format.
@@ -398,7 +596,32 @@ def _jwst_s3d_loader(filename, **kwargs):
             header.extend(slit_header, strip=True, update=True)
             meta = {'header': header}
 
-            spec = Spectrum1D(flux=flux, spectral_axis=wavelength, meta=meta)
+            # get uncertainty information
+            ext_name = primary_header.get("ERREXT", "ERR")
+            err_type = hdulist[ext_name].header.get("ERRTYPE", 'ERR')
+            err_unit = hdulist[ext_name].header.get("BUNIT", None)
+            err_array = hdulist[ext_name].data.T
+
+            # ERRTYPE can be one of "ERR", "IERR", "VAR", "IVAR"
+            # but mostly ERR for JWST cubes
+            # see https://jwst-pipeline.readthedocs.io/en/latest/jwst/data_products/science_products.html#s3d
+            if err_type == "ERR":
+                err = StdDevUncertainty(err_array, unit=err_unit)
+            elif err_type == 'VAR':
+                err = VarianceUncertainty(err_array, unit=err_unit)
+            elif err_type == 'IVAR':
+                err = InverseVariance(err_array, unit=err_unit)
+            elif err_type == 'IERR':
+                log.warning("Inverse Error is not yet a supported Astropy.nddata "
+                            "uncertainty. Setting err to None.")
+                err = None
+
+            # get mask information
+            mask_name = primary_header.get("MASKEXT", "DQ")
+            mask = hdulist[mask_name].data.T
+
+            spec = Spectrum1D(flux=flux, spectral_axis=wavelength, meta=meta,
+                              uncertainty=err, mask=mask)
             spectra.append(spec)
 
     return SpectrumList(spectra)
