@@ -7,88 +7,115 @@ import astropy.units as u
 
 class SpectralRegion:
     """
-    A `SpectralRegion` is a container class that enables some simplicty
-    in defining and passing a region (interval) for a spectrum.
+    A `SpectralRegion` is a container class for regions (intervals) along a
+    spectral coordinate.
 
-    In the future, there might be more functionality added in here, and there
-    is some discussion that this might/could move to
-    `Astropy Regions <http://astropy-regions.readthedocs.io/en/latest/>`_.
+    This class can either represent a single contiguous region or a set of
+    regions related to each other in some way (For example, a pair of
+    continuum windows around a line or a doublet of lines).
 
     Parameters
     ----------
+    *args : variable
+        Either a single parameter or two parameters can be given:
 
-    lower : Scalar `~astropy.units.Quantity` with pixel or any valid ``spectral_axis`` unit
-       The lower bound of the region.
-
-    upper : Scalar `~astropy.units.Quantity` with pixel or any valid ``spectral_axis`` unit
-       The upper bound of the region.
+        * 1 argument ``regioniter``: An iterable of length-2
+          `~astropy.units.Quantity` objects (or a single n x 2
+          `~astropy.units.Quantity` object), where the length-2 dimension is
+          ``lower``, ``upper``.
+        * ``lower``, ``upper``: Each should be an `~astropy.units.Quantity`
+          object.
 
     Notes
     -----
     The subregions will be ordered based on the lower bound of each subregion.
 
     """
+    def __init__(self, *args):
+        # Create instance variables
+        self._subregions = None
+
+        # Set the values (using the setters for doing the proper checking)
+        if len(args) == 1:
+            if all([self._is_2_element(x) for x in args[0]]):
+                self._subregions = [tuple(x) for x in args[0]]
+            else:
+                raise ValueError("SpectralRegion 1-argument input must be "
+                                 "a list of length-two Quantity objects.")
+        elif len(args) == 2:
+            if self._is_2_element(args):
+                self._subregions = [tuple(args)]
+            else:
+                raise ValueError("SpectralRegion 2-argument inputs must be "
+                                 "Quantity objects.")
+        else:
+            raise TypeError(f'SpectralRegion initializer takes 1 or 2'
+                            f'positional arguments but {len(args)} were given')
+
+        #  Check validity of the input sub regions.
+        if not self._valid():
+            raise ValueError("SpectralRegion 2-tuple lower extent must be "
+                             "less than upper extent.")
+
+        # The sub-regions are to always be ordered based on the lower bound.
+        self._reorder()
 
     @classmethod
     def from_center(cls, center=None, width=None):
         """
-        SpectralRegion class method that enables the definition of a `SpectralRegion`
-        from the center and width rather than lower and upper bounds.
+        SpectralRegion class method that enables the definition of a
+        `SpectralRegion` from the center and width rather than lower and
+        upper bounds.
 
         Parameters
         ----------
-
         center : Scalar `~astropy.units.Quantity` with pixel or any valid ``spectral_axis`` unit
            The center of the spectral region.
-
         width : Scalar `~astropy.units.Quantity` with pixel or any valid ``spectral_axis`` unit
            The full width of the spectral region (upper bound - lower bound).
         """
 
         if width.value <= 0:
-            raise ValueError('SpectralRegion width must be positive.')
+            raise ValueError("SpectralRegion width must be positive.")
 
         if center.unit.physical_type != 'length':
             return cls(center + width/2, center - width/2)
 
         return cls(center - width/2, center + width/2)
 
-    def __init__(self, *args):
+    @classmethod
+    def from_line_list(cls, table, width=1):
         """
-        Lower and upper values for the interval.
+        Generate a ``SpectralRegion`` instance from the `~astropy.table.QTable`
+        object returned from `~specutils.fitting.find_lines_derivative` or
+        `~specutils.fitting.find_lines_threshold`.
+
+        Parameters
+        ----------
+        table : `~astropy.table.QTable`
+            List of found lines.
+        width : float
+            The width of the spectral line region. If not unit information is
+            provided, it's assumed to be the same units as used in the line
+            list table.
+
+        Returns
+        -------
+        `~specutils.SpectralRegion`
+            The spectral region based on the line list.
         """
+        width = u.Quantity(width, table['line_center'].unit)
 
-        # Create instance variables
-        self._subregions = None
-
-        #
-        # Set the values (using the setters for doing the proper checking)
-        #
-
-        if self._is_2_element(args):
-            self._subregions = [tuple(args)]
-        elif isinstance(args, (list, tuple)) and all([self._is_2_element(x) for x in args[0]]):
-            self._subregions = [tuple(x) for x in args[0]]
-        else:
-            raise ValueError('SpectralRegion input must be a 2-tuple or a list of 2-tuples.')
-
-        #
-        #  Check validity of the input sub regions.
-        #
-        if not self._valid():
-            raise ValueError('SpectralRegion 2-tuple lower extent must be less than upper extent.')
-
-        #
-        # The sub-regions are to always be ordered based on the lower bound.
-        #
-        self._reorder()
+        return cls([(x - width * 0.5, x + width * 0.5)
+                    for x in table['line_center']])
 
     def _info(self):
         """
         Pretty print the sub-regions.
         """
 
-        toreturn = 'Spectral Region, {} sub-regions:\n'.format(len(self._subregions))
+        toreturn = "Spectral Region, {} sub-regions:\n".format(
+            len(self._subregions))
 
         # Setup the subregion text.
         subregion_text = []
@@ -168,10 +195,11 @@ class SpectralRegion:
 
         return True
 
-    def _is_2_element(self, value):
+    @staticmethod
+    def _is_2_element(value):
         """
         Helper function to check a variable to see if it
-        is a 2-tuple.
+        is a 2-tuple of Quantity objects.
         """
         return len(value) == 2 and \
                isinstance(value[0], u.Quantity) and \
@@ -185,6 +213,10 @@ class SpectralRegion:
 
     @property
     def subregions(self):
+        """
+        An iterable over ``(lower, upper)`` tuples that are each of the
+        sub-regions.
+        """
         return self._subregions
 
     @property
@@ -235,15 +267,14 @@ class SpectralRegion:
         object defines, create a new `SpectralRegion` such that the sub-regions
         are defined in the new one as regions *not* in this `SpectralRegion`.
 
-
         Parameters
         ----------
-
-        lower_bound : Scalar `~astropy.units.Quantity` with pixel or any valid ``spectral_axis`` unit
-           The lower bound of the region.
-
-        upper_bound : Scalar `~astropy.units.Quantity` with pixel or any valid ``spectral_axis`` unit
-           The upper bound of the region.
+        lower_bound : `~astropy.units.Quantity`
+           The lower bound of the region. Can be scalar with pixel or any
+           valid ``spectral_axis`` unit
+        upper_bound : `~astropy.units.Quantity`
+           The upper bound of the region. Can be scalar with pixel or any
+           valid ``spectral_axis`` unit
 
         Returns
         -------
