@@ -1,20 +1,24 @@
-import logging
 from copy import deepcopy
+import logging
 
 import numpy as np
 from astropy import units as u
-from astropy.nddata import NDDataRef
 from astropy.utils.decorators import lazyproperty
-from .spectrum_mixin import OneDSpectrumMixin
+from astropy.nddata import NDUncertainty, NDIOMixin, NDArithmeticMixin
+
 from .spectral_axis import SpectralAxis
+from .spectrum_mixin import OneDSpectrumMixin
+from .spectral_region import SpectralRegion
 from ..utils.wcs_utils import gwcs_from_array
+from astropy.coordinates import SpectralCoord
+from ndcube import NDCube
 
 __all__ = ['Spectrum1D']
 
-__doctest_skip__ = ['Spectrum1D.spectral_resolution']
+log = logging.getLogger(__name__)
 
 
-class Spectrum1D(OneDSpectrumMixin, NDDataRef):
+class Spectrum1D(OneDSpectrumMixin, NDCube, NDIOMixin, NDArithmeticMixin):
     """
     Spectrum container for 1D spectral data.
 
@@ -33,13 +37,15 @@ class Spectrum1D(OneDSpectrumMixin, NDDataRef):
     Parameters
     ----------
     flux : `~astropy.units.Quantity` or `~astropy.nddata.NDData`-like
-        The flux data for this spectrum.
+        The flux data for this spectrum. This can be a simple `~astropy.units.Quantity`,
+        or an existing `~Spectrum1D` or `~ndcube.NDCube` object.
     spectral_axis : `~astropy.units.Quantity` or `~specutils.SpectralAxis`
         Dispersion information with the same shape as the last (or only)
         dimension of flux, or one greater than the last dimension of flux
         if specifying bin edges.
     wcs : `~astropy.wcs.WCS` or `~gwcs.wcs.WCS`
-        WCS information object.
+        WCS information object that either has a spectral component or is
+        only spectral.
     velocity_convention : {"doppler_relativistic", "doppler_optical", "doppler_radio"}
         Convention used for velocity conversions.
     rest_value : `~astropy.units.Quantity`
@@ -70,18 +76,31 @@ class Spectrum1D(OneDSpectrumMixin, NDDataRef):
                  radial_velocity=None, bin_specification=None, **kwargs):
         # Check for pre-defined entries in the kwargs dictionary.
         unknown_kwargs = set(kwargs).difference(
-            {'data', 'unit', 'uncertainty', 'meta', 'mask', 'copy'})
+            {'data', 'unit', 'uncertainty', 'meta', 'mask', 'copy',
+             'extra_coords'})
 
         if len(unknown_kwargs) > 0:
             raise ValueError("Initializer contains unknown arguments(s): {}."
                              "".format(', '.join(map(str, unknown_kwargs))))
 
-        # If the flux (data) argument is a subclass of nddataref (as it would
+        # If the flux (data) argument is already a Spectrum1D (as it would
         # be for internal arithmetic operations), avoid setup entirely.
-        if isinstance(flux, NDDataRef):
+        if isinstance(flux, Spectrum1D):
             super().__init__(flux)
             return
 
+        # Handle initializing from NDCube objects
+        elif isinstance(flux, NDCube):
+            if flux.unit is None:
+                raise ValueError("Input NDCube missing unit parameter")
+
+            # Change the flux array from bare ndarray to a Quantity
+            q_flux = flux.data << u.Unit(flux.unit)
+
+            self.__init__(flux=q_flux, wcs=flux.wcs, mask=flux.mask,
+                          uncertainty=flux.uncertainty)
+            return
+
         # If the mask kwarg is not passed to the constructor, but the flux array
         # contains NaNs, add the NaN locations to the mask.
         if "mask" not in kwargs and flux is not None:
@@ -137,7 +156,7 @@ class Spectrum1D(OneDSpectrumMixin, NDDataRef):
                 rest_value = None
         else:
             if not isinstance(rest_value, u.Quantity):
-                logging.info("No unit information provided with rest value. "
+                log.info("No unit information provided with rest value. "
                              "Assuming units of spectral axis ('%s').",
                              spectral_axis.unit)
                 rest_value = u.Quantity(rest_value, spectral_axis.unit)
@@ -162,8 +181,51 @@ class Spectrum1D(OneDSpectrumMixin, NDDataRef):
                 raise ValueError(
                     "Spectral axis length ({}) must be the same size or one "
                     "greater (if specifying bin edges) than that of the last "
-                    "flux axis ({})".format(spectral_axis.shape[0], \
-                        flux.shape[-1]))
+                    "flux axis ({})".format(spectral_axis.shape[0],
+                                            flux.shape[-1]))
+
+        # If a WCS is provided, check that the spectral axis is last and reorder
+        # the arrays if not
+        if wcs is not None and hasattr(wcs, "naxis"):
+            if wcs.naxis > 1:
+                temp_axes = []
+                phys_axes = wcs.world_axis_physical_types
+                for i in range(len(phys_axes)):
+                    if phys_axes[i] is None:
+                        continue
+                    if phys_axes[i][0:2] == "em" or phys_axes[i][0:5] == "spect":
+                        temp_axes.append(i)
+                if len(temp_axes) != 1:
+                    raise ValueError("Input WCS must have exactly one axis with "
+                                     "spectral units, found {}".format(len(temp_axes)))
+
+                # Due to FITS conventions, a WCS with spectral axis first corresponds
+                # to a flux array with spectral axis last.
+                if temp_axes[0] != 0:
+                    log.warn("Input WCS indicates that the spectral axis is not"
+                             " last. Reshaping arrays to put spectral axis last.")
+                    wcs = wcs.swapaxes(0, temp_axes[0])
+                    if flux is not None:
+                        flux = np.swapaxes(flux, len(flux.shape)-temp_axes[0]-1, -1)
+                    if "mask" in kwargs:
+                        if kwargs["mask"] is not None:
+                            kwargs["mask"] = np.swapaxes(kwargs["mask"],
+                                                len(kwargs["mask"].shape)-temp_axes[0]-1, -1)
+                    if "uncertainty" in kwargs:
+                        if kwargs["uncertainty"] is not None:
+                            if isinstance(kwargs["uncertainty"], NDUncertainty):
+                                # Account for Astropy uncertainty types
+                                unc_len = len(kwargs["uncertainty"].array.shape)
+                                temp_unc = np.swapaxes(kwargs["uncertainty"].array,
+                                                       unc_len-temp_axes[0]-1, -1)
+                                if kwargs["uncertainty"].unit is not None:
+                                    temp_unc = temp_unc * u.Unit(kwargs["uncertainty"].unit)
+                                kwargs["uncertainty"] = type(kwargs["uncertainty"])(temp_unc)
+                            else:
+                                kwargs["uncertainty"] = np.swapaxes(kwargs["uncertainty"],
+                                                        len(kwargs["uncertainty"].shape) -
+                                                        temp_axes[0]-1, -1)
+
 
         # Attempt to parse the spectral axis. If none is given, try instead to
         # parse a given wcs. This is put into a GWCS object to
@@ -197,7 +259,9 @@ class Spectrum1D(OneDSpectrumMixin, NDDataRef):
 
                 self._spectral_axis = spectral_axis
 
-            wcs = gwcs_from_array(self._spectral_axis)
+            if wcs is None:
+                wcs = gwcs_from_array(self._spectral_axis)
+
         elif wcs is None:
             # If no spectral axis or wcs information is provided, initialize
             # with an empty gwcs based on the flux.
@@ -212,12 +276,23 @@ class Spectrum1D(OneDSpectrumMixin, NDDataRef):
         # If no spectral_axis was provided, create a SpectralCoord based on
         # the WCS
         if spectral_axis is None:
-            # If spectral_axis wasn't provided, set _spectral_axis based on
-            # the WCS
-            spec_axis = self.wcs.pixel_to_world(np.arange(self.flux.shape[-1]))
+            # If the WCS doesn't have a spectral attribute, we assume it's the
+            # dummy GWCS we created or a solely spectral WCS
+            if hasattr(self.wcs, "spectral"):
+                # Handle generated 1D WCS that aren't set to spectral
+                if not self.wcs.is_spectral and self.wcs.naxis == 1:
+                    spec_axis = self.wcs.pixel_to_world(np.arange(self.flux.shape[-1]))
+                else:
+                    spec_axis = self.wcs.spectral.pixel_to_world(np.arange(self.flux.shape[-1]))
+            else:
+                spec_axis = self.wcs.pixel_to_world(np.arange(self.flux.shape[-1]))
 
-            if spec_axis.unit.is_equivalent(u.one):
-                spec_axis = spec_axis * u.pixel
+            try:
+                if spec_axis.unit.is_equivalent(u.one):
+                    spec_axis = spec_axis * u.pixel
+            except AttributeError:
+                raise AttributeError(f"spec_axis does not have unit: "
+                                     f"{type(spec_axis)} {spec_axis}")
 
             self._spectral_axis = SpectralAxis(
                 spec_axis,
@@ -246,15 +321,70 @@ class Spectrum1D(OneDSpectrumMixin, NDDataRef):
         The first case is handled by the parent class, while the second is
         handled here.
         """
-        if len(self.flux.shape) > 1:
+
+        if self.flux.ndim > 1 or (type(item) == tuple and item[0] == Ellipsis):
+            if type(item) == tuple:
+                if len(item) == len(self.flux.shape) or item[0] == Ellipsis:
+                    spec_item = item[-1]
+                    if not isinstance(spec_item, slice):
+                        if isinstance(item, u.Quantity):
+                            raise ValueError("Indexing on single spectral axis "
+                                             "values is not currently allowed, "
+                                             "please use a slice.")
+                        spec_item = slice(spec_item, spec_item+1, None)
+                        item = item[:-1] + (spec_item,)
+                else:
+                    # Slicing on less than the full number of axes means we want
+                    # to keep the whole spectral axis
+                    spec_item = slice(None, None, None)
+            elif isinstance(item, slice) and (isinstance(item.start, u.Quantity) or
+                    isinstance(item.stop, u.Quantity)):
+                # We only allow slicing with world coordinates along the spectral
+                # axis for now
+                for attr in ("start", "stop"):
+                    if getattr(item, attr) is None:
+                        continue
+                    if not getattr(item, attr).unit.is_equivalent(u.AA,
+                            equivalencies=u.spectral()):
+                        raise ValueError("Slicing with world coordinates is only"
+                                         " enabled for spectral coordinates.")
+                        break
+                spec_item = item
+            else:
+                # Slicing with a single integer or slice uses the leading axis,
+                # so we keep the whole spectral axis, which is last
+                spec_item = slice(None, None, None)
+
+            if (isinstance(spec_item.start, u.Quantity) or
+                    isinstance(spec_item.stop, u.Quantity)):
+                temp_spec = self._spectral_slice(spec_item)
+                if spec_item is item:
+                    return temp_spec
+                else:
+                    # Drop the spectral axis slice and perform only the spatial part
+                    return temp_spec[item[:-1]]
+
             return self._copy(
                 flux=self.flux[item],
+                spectral_axis=self.spectral_axis[spec_item],
                 uncertainty=self.uncertainty[item]
                 if self.uncertainty is not None else None,
                 mask=self.mask[item] if self.mask is not None else None)
 
         if not isinstance(item, slice):
-            item = slice(item, item+1, None)
+            if isinstance(item, u.Quantity):
+                raise ValueError("Indexing on a single spectral axis values is not"
+                                 " currently allowed, please use a slice.")
+            # Handle tuple slice as input by NDCube crop method
+            elif isinstance(item, tuple):
+                if len(item) == 1 and isinstance(item[0], slice):
+                    item = item[0]
+                else:
+                    raise ValueError(f"Unclear how to slice with tuple {item}")
+            else:
+                item = slice(item, item + 1, None)
+        elif (isinstance(item.start, u.Quantity) or isinstance(item.stop, u.Quantity)):
+            return self._spectral_slice(item)
 
         tmp_spec = super().__getitem__(item)
 
@@ -272,7 +402,7 @@ class Spectrum1D(OneDSpectrumMixin, NDDataRef):
 
     def _copy(self, **kwargs):
         """
-        Peform deep copy operations on each attribute of the ``Spectrum1D``
+        Perform deep copy operations on each attribute of the ``Spectrum1D``
         object.
         """
         alt_kwargs = dict(
@@ -290,6 +420,39 @@ class Spectrum1D(OneDSpectrumMixin, NDDataRef):
 
         return self.__class__(**alt_kwargs)
 
+    def _spectral_slice(self, item):
+        """
+        Perform a region extraction given a slice on the spectral axis.
+        """
+        from ..manipulation import extract_region
+        if item.start is None:
+            start = self.spectral_axis[0]
+        else:
+            start = item.start
+        if item.stop is None:
+            stop = self.spectral_axis[-1]
+        else:
+            # Force the upper bound to be open, as in normal python array slicing
+            exact_match = np.where(self.spectral_axis == item.stop)
+            if len(exact_match[0]) == 1:
+                stop_index = exact_match[0][0] - 1
+                stop = self.spectral_axis[stop_index]
+            else:
+                stop = item.stop
+        reg = SpectralRegion(start, stop)
+        return extract_region(self, reg)
+
+    @NDCube.mask.setter
+    def mask(self, value):
+        # Impose stricter checks than the base NDData mask setter
+        if value is not None:
+            value = np.array(value)
+            if not self.data.shape == value.shape:
+                raise ValueError(
+                    "Flux axis ({}) and mask ({}) shapes must be the "
+                    "same.".format(self.data.shape, value.shape))
+        self._mask = value
+
     @property
     def frequency(self):
         """
@@ -348,7 +511,8 @@ class Spectrum1D(OneDSpectrumMixin, NDDataRef):
 
     @redshift.setter
     def redshift(self, val):
-        new_spec_coord = self.spectral_axis.with_redshift(val)
+        new_spec_coord = self.spectral_axis.with_radial_velocity_shift(
+            -self.spectral_axis.radial_velocity).with_radial_velocity_shift(val)
         self._spectral_axis = new_spec_coord
 
     @property
@@ -369,35 +533,36 @@ class Spectrum1D(OneDSpectrumMixin, NDDataRef):
             if not val.unit.is_equivalent(u.km/u.s):
                 raise u.UnitsError("Radial velocity must be a velocity.")
 
-        new_spectral_axis = self.spectral_axis.with_radial_velocity(val)
+        new_spectral_axis = self.spectral_axis.with_radial_velocity_shift(
+            -self.spectral_axis.radial_velocity).with_radial_velocity_shift(val)
         self._spectral_axis = new_spectral_axis
 
     def __add__(self, other):
-        if not isinstance(other, NDDataRef):
+        if not isinstance(other, NDCube):
             other = u.Quantity(other, unit=self.unit)
 
         return self.add(other)
 
     def __sub__(self, other):
-        if not isinstance(other, NDDataRef):
+        if not isinstance(other, NDCube):
             other = u.Quantity(other, unit=self.unit)
 
         return self.subtract(other)
 
     def __mul__(self, other):
-        if not isinstance(other, NDDataRef):
+        if not isinstance(other, NDCube):
             other = u.Quantity(other)
 
         return self.multiply(other)
 
     def __div__(self, other):
-        if not isinstance(other, NDDataRef):
+        if not isinstance(other, NDCube):
             other = u.Quantity(other)
 
         return self.divide(other)
 
     def __truediv__(self, other):
-        if not isinstance(other, NDDataRef):
+        if not isinstance(other, NDCube):
             other = u.Quantity(other)
 
         return self.divide(other)
