@@ -1,11 +1,10 @@
-import logging
 import os
 import sys
 import shutil
 import tempfile
 import urllib
 import warnings
-
+import uuid
 import pytest
 
 import astropy.units as u
@@ -23,9 +22,10 @@ from astropy.nddata import StdDevUncertainty
 from numpy.testing import assert_allclose
 
 from .conftest import remote_access
-from .. import Spectrum1D, SpectrumList
+from .. import Spectrum1D, SpectrumCollection, SpectrumList
 from ..io import get_loaders_by_extension
 from ..io.default_loaders import subaru_pfs_spec
+from ..io.default_loaders.sdss import _sdss_wcs_to_log_wcs
 
 # NOTE: Python can be built without bz2 or lzma.
 try:
@@ -43,6 +43,9 @@ else:
     HAS_LZMA = True
 
 
+EBOSS_SPECTRUM_URL = 'https://data.sdss.org/sas/dr16/eboss/spectro/redux/v5_13_0/spectra/lite/4055/spec-4055-55359-0596.fits'
+
+
 def test_get_loaders_by_extension():
     loader_labels = get_loaders_by_extension('fits')
 
@@ -169,39 +172,45 @@ def test_manga_rss():
 
 @pytest.mark.remote_data
 def test_sdss_spec():
-    sp_pattern = 'spec-4055-55359-0596.fits.'
-    with urllib.request.urlopen('https://dr14.sdss.org/optical/spectrum/view/data/format%3Dfits/spec%3Dlite?mjd=55359&fiberid=596&plateid=4055') as response:
+    sp_pattern = 'spec-4055-55359-0596.fits'
+    with urllib.request.urlopen(EBOSS_SPECTRUM_URL) as response:
         # Read from open file object
         spec = Spectrum1D.read(response, format="SDSS-III/IV spec")
         assert isinstance(spec, Spectrum1D)
         assert spec.flux.size > 0
 
-    with urllib.request.urlopen('https://dr14.sdss.org/optical/spectrum/view/data/format%3Dfits/spec%3Dlite?mjd=55359&fiberid=596&plateid=4055') as response:
-        with tempfile.NamedTemporaryFile() as tmp_file:
-            shutil.copyfileobj(response, tmp_file)
+    with urllib.request.urlopen(EBOSS_SPECTRUM_URL) as response:
+        # On Windows, NamedTemporaryFile cannot be opened a second time while
+        #  already being open, so we avoid using that method.
+        with tempfile.TemporaryDirectory() as tmp_dir:
+            file_path = os.path.join(tmp_dir, sp_pattern)
 
-            # Read from local disk via file signature
-            spec = Spectrum1D.read(tmp_file.name)
+            with open(file_path, 'wb') as tmp_file:
+                shutil.copyfileobj(response, tmp_file)
 
-            assert isinstance(spec, Spectrum1D)
-            assert spec.flux.size > 0
+                # Read from local disk via filename
+                print(tmp_file.name)
+                spec = Spectrum1D.read(tmp_file.name)
 
-            # Read from HDUList object
-            hdulist = fits.open(tmp_file.name)
-            spec = Spectrum1D.read(hdulist)
-            assert isinstance(spec, Spectrum1D)
-            assert spec.flux.size > 0
+                assert isinstance(spec, Spectrum1D)
+                assert spec.flux.size > 0
 
-            # Read from file handle
-            fileio = open(tmp_file.name, mode='rb')
-            spec = Spectrum1D.read(fileio)
-            assert isinstance(spec, Spectrum1D)
-            assert spec.flux.size > 0
+                # Read from HDUList object
+                hdulist = fits.open(tmp_file.name)
+                spec = Spectrum1D.read(hdulist)
+                assert isinstance(spec, Spectrum1D)
+                assert spec.flux.size > 0
+
+                # Read from file handle
+                fileio = open(tmp_file.name, mode='rb')
+                spec = Spectrum1D.read(fileio)
+                assert isinstance(spec, Spectrum1D)
+                assert spec.flux.size > 0
 
 
 @pytest.mark.remote_data
 def test_sdss_spspec():
-    sp_pattern = 'spSpec-51957-0273-016.fit.'
+    sp_pattern = 'spSpec-51957-0273-016.fit'
     with urllib.request.urlopen('http://das.sdss.org/spectro/1d_26/0273/1d/spSpec-51957-0273-016.fit') as response:
         # Read from open file object
         spec = Spectrum1D.read(response, format="SDSS-I/II spSpec")
@@ -209,28 +218,32 @@ def test_sdss_spspec():
         assert spec.flux.size > 0
 
     with urllib.request.urlopen('http://das.sdss.org/spectro/1d_26/0273/1d/spSpec-51957-0273-016.fit') as response:
-        with tempfile.NamedTemporaryFile() as tmp_file:
-            shutil.copyfileobj(response, tmp_file)
+        # On Windows, NamedTemporaryFile cannot be opened a second time while
+        #  already being open, so we avoid using that method.
+        with tempfile.TemporaryDirectory() as tmp_dir:
+            file_path = os.path.join(tmp_dir, sp_pattern)
 
-            # Read from local disk via file signature
-            with warnings.catch_warnings():
-                warnings.simplefilter('ignore', FITSFixedWarning)
-                spec = Spectrum1D.read(tmp_file.name)
+            with open(file_path, 'wb') as tmp_file:
+                shutil.copyfileobj(response, tmp_file)
 
-            assert isinstance(spec, Spectrum1D)
-            assert spec.flux.size > 0
+                with warnings.catch_warnings():
+                    warnings.simplefilter('ignore', FITSFixedWarning)
+                    spec = Spectrum1D.read(tmp_file.name)
 
-            # Read from HDUList object
-            hdulist = fits.open(tmp_file.name)
-            spec = Spectrum1D.read(hdulist)
-            assert isinstance(spec, Spectrum1D)
-            assert spec.flux.size > 0
+                assert isinstance(spec, Spectrum1D)
+                assert spec.flux.size > 0
 
-            # Read from file handle
-            fileio = open(tmp_file.name, mode='rb')
-            spec = Spectrum1D.read(fileio)
-            assert isinstance(spec, Spectrum1D)
-            assert spec.flux.size > 0
+                # Read from HDUList object
+                hdulist = fits.open(tmp_file.name)
+                spec = Spectrum1D.read(hdulist)
+                assert isinstance(spec, Spectrum1D)
+                assert spec.flux.size > 0
+
+                # Read from file handle
+                fileio = open(tmp_file.name, mode='rb')
+                spec = Spectrum1D.read(fileio)
+                assert isinstance(spec, Spectrum1D)
+                assert spec.flux.size > 0
 
 
 @pytest.mark.remote_data
@@ -238,8 +251,7 @@ def test_sdss_spec_stream():
     """Test direct read and recognition of SDSS-III/IV spec from remote URL,
     i.e. do not rely on filename pattern.
     """
-    sdss_url = 'https://dr14.sdss.org/optical/spectrum/view/data/format%3Dfits/spec%3Dlite?mjd=55359&fiberid=596&plateid=4055'
-    spec = Spectrum1D.read(sdss_url)
+    spec = Spectrum1D.read(EBOSS_SPECTRUM_URL)
 
     assert isinstance(spec, Spectrum1D)
     assert spec.flux.size > 0
@@ -438,6 +450,43 @@ def test_iraf_non_linear_cubic_spline(remote_data_path):
         assert Spectrum1D.read(remote_data_path, format='iraf')
 
 
+@pytest.mark.remote_data
+def test_iraf_multispec_chebyshev():
+    """Test loading of SpectrumCollection from IRAF MULTISPEC format FITS file -
+    nonlinear 2D WCS with Chebyshev solution (FTYPE=1).
+    """
+    iraf_url = 'https://github.com/astropy/specutils/raw/legacy-specutils/specutils/io/tests/files'
+    # Read reference ASCII spectrum from remote file.
+    spec10 = Table.read(iraf_url + '/AAO_11.txt', format='ascii.no_header',
+                        data_start=175, names=['spectral_axis', 'flux'])
+
+    # Read full collection of 51 spectra from open FITS file object
+    speccol = SpectrumCollection.read(iraf_url + '/AAO.fits')
+
+    assert len(speccol) == 51
+    # Numpy allclose does support quantities, but not with unit 'adu'!
+    assert quantity_allclose(speccol[10].spectral_axis, spec10['spectral_axis'] * u.AA)
+    assert quantity_allclose(speccol[10].flux, spec10['flux'] * u.adu)
+
+
+@pytest.mark.remote_data
+def test_iraf_multispec_legendre():
+    """Test loading of SpectrumCollection from IRAF MULTISPEC format FITS file -
+    nonlinear 2D WCS with Legendre solution (FTYPE=2).
+    """
+    iraf_url = 'https://github.com/astropy/specutils/raw/legacy-specutils/specutils/io/tests/files'
+    # Read reference ASCII spectrum from remote file.
+    spec10 = Table.read(iraf_url + '/TRES.dat', format='ascii.no_header',
+                        data_start=127, names=['spectral_axis', 'flux'])
+
+    # Read full collection of 51 spectra from remote FITS file object
+    speccol = SpectrumCollection.read(iraf_url + '/TRES.fits')
+
+    assert len(speccol) == 51
+    # The reference spectrum flux is normalised/flatfielded, just check the wavelength solution
+    assert_allclose(speccol[10].spectral_axis, spec10['spectral_axis'] * u.AA)
+
+
 @pytest.mark.parametrize("spectral_axis",
                          ['wavelength', 'frequency', 'energy', 'wavenumber'])
 def test_tabular_fits_writer(tmpdir, spectral_axis):
@@ -473,14 +522,15 @@ def test_tabular_fits_writer(tmpdir, spectral_axis):
         spectrum.write(tmpfile, format='tabular-fits')
     spectrum.write(tmpfile, format='tabular-fits', overwrite=True)
 
-    cmap = {spectral_axis: ('spectral_axis', wlu[spectral_axis]),
+    # Map to alternative set of units
+    cmap = {spectral_axis: ('spectral_axis', 'micron'),
             'flux': ('flux', 'erg / (s cm**2 AA)'),
             'uncertainty': ('uncertainty', None)}
 
     # Read it back again and check against the original
     spec = Spectrum1D.read(tmpfile, format='tabular-fits', column_mapping=cmap)
     assert spec.flux.unit == u.Unit('erg / (s cm**2 AA)')
-    assert spec.spectral_axis.unit == spectrum.spectral_axis.unit
+    assert spec.spectral_axis.unit == u.um
     assert quantity_allclose(spec.spectral_axis, spectrum.spectral_axis)
     assert quantity_allclose(spec.flux, spectrum.flux)
     assert quantity_allclose(spec.uncertainty.quantity,
@@ -516,14 +566,14 @@ def test_tabular_fits_multid(tmpdir, ndim, spectral_axis):
     assert quantity_allclose(spec.uncertainty.quantity,
                              spectrum.uncertainty.quantity)
 
-    # Test again, using `column_mapping` to convert to different flux unit
-    cmap = {spectral_axis: ('spectral_axis', wlu[spectral_axis]),
+    # Test again, using `column_mapping` to convert to different spectral axis and flux units
+    cmap = {spectral_axis: ('spectral_axis', 'THz'),
             'flux': ('flux', 'erg / (s cm**2 AA)'),
             'uncertainty': ('uncertainty', None)}
 
     spec = Spectrum1D.read(tmpfile, format='tabular-fits', column_mapping=cmap)
     assert spec.flux.unit == u.Unit('erg / (s cm**2 AA)')
-    assert spec.spectral_axis.unit == spectrum.spectral_axis.unit
+    assert spec.spectral_axis.unit == u.THz
     assert quantity_allclose(spec.spectral_axis, spectrum.spectral_axis)
     assert quantity_allclose(spec.flux, spectrum.flux)
     assert quantity_allclose(spec.uncertainty.quantity,
@@ -980,7 +1030,7 @@ def test_spectrum1d_6dfgs_split_combined(remote_data_path):
 
 
 @remote_access([{'id': '3733958', 'filename': 'all-c0022498-344732.fits'}])
-def test_spectrum1d_6dfgs_split_combined(remote_data_path):
+def test_spectrum1d_6dfgs_combined(remote_data_path):
     specs = SpectrumList.read(remote_data_path)
 
     for spec in specs:
@@ -996,6 +1046,8 @@ def test_spectrum1d_6dfgs_split_combined(remote_data_path):
         assert isinstance(spec, Spectrum1D)
         assert spec.flux.unit == u.Unit("count/Angstrom")
         assert spec.flux.size > 0
+        assert spec.meta["sky"].flux.unit == u.Unit("count/Angstrom")
+        assert spec.meta["sky"].flux.size > 0
 
     assert len(specs) == 3
 
@@ -1076,3 +1128,316 @@ def test_spectrum_list_2dfgrs_multiple(remote_data_path):
     assert len(specs) == 2
 
     hdulist.close()
+
+
+def test_sdss_wcs_handler():
+    sdss_wcs = WCS(naxis=2)
+    sdss_wcs.wcs.crval[0] = 3.57880000000000E+00
+    sdss_wcs.wcs.cd = [[1.00000000000000E-04, 0], [0, 1]]
+    sdss_wcs.wcs.cunit[0] = u.Unit('Angstrom')
+    fixed_wcs = _sdss_wcs_to_log_wcs(sdss_wcs)
+    dropped_sdss_wcs = sdss_wcs.dropaxis(1)
+    dropped_sdss_wcs.wcs.cunit[0] = ''  # Cannot handle units in powers
+    sdss_wave = 10 ** dropped_sdss_wcs.pixel_to_world(np.arange(10)) * u.Unit('Angstrom')
+    fixed_wave = fixed_wcs.pixel_to_world(np.arange(10))
+    assert quantity_allclose(sdss_wave, fixed_wave)
+
+
+class TestAAOmega2dF:
+    @remote_access([{'id': '4460981', 'filename': "OBJ0039red.fits"}])
+    def test_with_rwss(self, remote_data_path):
+        spectra = SpectrumList.read(
+            remote_data_path, format="Data Central AAOmega",
+        )
+        assert len(spectra) == 139
+        for spec in spectra:
+            assert spec.meta.get("label") is not None
+            assert spec.meta.get("header") is not None
+            assert spec.meta.get("purpose") is not None
+            assert spec.meta.get("fibre_index") is not None
+
+    @remote_access([{'id': '4460981', 'filename': "OBJ0032red.fits"}])
+    def test_without_rwss(self, remote_data_path):
+        spectra = SpectrumList.read(
+            remote_data_path, format="Data Central AAOmega",
+        )
+        assert len(spectra) == 153
+        for spec in spectra:
+            assert spec.meta.get("label") is not None
+            assert spec.meta.get("header") is not None
+            assert spec.meta.get("purpose") is not None
+            assert spec.meta.get("fibre_index") is not None
+
+    @remote_access([{'id': '4460981', 'filename': "OBJ0039red.fits"}])
+    def test_with_rwss_guess(self, remote_data_path):
+        spectra = SpectrumList.read(remote_data_path)
+        assert len(spectra) == 139
+        for spec in spectra:
+            assert spec.meta.get("label") is not None
+            assert spec.meta.get("header") is not None
+            assert spec.meta.get("purpose") is not None
+            assert spec.meta.get("fibre_index") is not None
+
+    @remote_access([{'id': '4460981', 'filename': "OBJ0032red.fits"}])
+    def test_without_rwss_guess(self, remote_data_path):
+        spectra = SpectrumList.read(remote_data_path)
+        assert len(spectra) == 153
+        for spec in spectra:
+            assert spec.meta.get("label") is not None
+            assert spec.meta.get("header") is not None
+            assert spec.meta.get("purpose") is not None
+            assert spec.meta.get("fibre_index") is not None
+
+
+@remote_access([
+    {'id': "4460981", 'filename':"1812260046012353.fits"}, # 4 exts
+    {'id': "4460981", 'filename':"1311160005010021.fits"} # 5 exts
+])
+def test_galah(remote_data_path):
+    spectra = SpectrumList.read(remote_data_path, format="GALAH")
+    # Should be main spectra, without sky, and normalised (not in 4 ext)
+    nspec = len(spectra)
+    if spectra[0].meta["galah_hdu_format"] == 4:
+        assert nspec == 2
+    elif spectra[0].meta["galah_hdu_format"] == 5:
+        assert nspec == 3
+    else:
+        assert False, "Unknown format"
+    # normalised
+    if nspec == 3:
+        assert spectra[0].flux.unit == u.Unit('') # dimensionless
+        assert spectra[0].spectral_axis.unit == u.Angstrom
+        assert spectra[0].uncertainty is None
+        assert spectra[0].meta.get("label") == "normalised spectra"
+        assert spectra[0].meta.get("header") is not None
+
+        # drop the normalised spectra, so 4 and 5 should now look the same
+        spectra = spectra[1:]
+
+    # main spectra
+    assert spectra[0].flux.unit == u.count
+    assert spectra[0].spectral_axis.unit == u.Angstrom
+    assert isinstance(spectra[0].uncertainty, StdDevUncertainty)
+    assert spectra[0].meta.get("label") is not None
+    assert spectra[0].meta.get("header") is not None
+
+    # No sky
+    assert spectra[1].spectral_axis.unit == u.Angstrom
+    assert spectra[1].flux.unit == u.count
+    assert isinstance(spectra[1].uncertainty, StdDevUncertainty)
+    assert spectra[1].meta.get("label") is not None
+    assert spectra[1].meta.get("header") is not None
+
+
+@pytest.mark.xfail(reason="Format is ambiguous")
+@remote_access([
+    {'id': "4460981", 'filename':"1812260046012353.fits"}, # 4 exts
+    {'id': "4460981", 'filename':"1311160005010021.fits"} # 5 exts
+])
+def test_galah_guess(remote_data_path):
+    spectra = SpectrumList.read(remote_data_path)
+    # Should be main spectra, without sky, and normalised (not in 4 ext)
+    nspec = len(spectra)
+    if spectra[0].meta["galah_hdu_format"] == 4:
+        assert nspec == 2
+    elif spectra[0].meta["galah_hdu_format"] == 5:
+        assert nspec == 3
+    else:
+        assert False, "Unknown format"
+
+    # main spectra
+    assert spectra[0].flux.unit == u.count
+    assert spectra[0].spectral_axis.unit == u.Angstrom
+    assert isinstance(spectra[0].uncertainty, StdDevUncertainty)
+    assert spectra[0].meta.get("label") is not None
+    assert spectra[0].meta.get("header") is not None
+
+    # normalised
+    if nspec == 3:
+        assert spectra[1].flux.unit == u.Unit('') # dimensionless
+        assert spectra[1].spectral_axis.unit == u.Angstrom
+        assert spectra[1].uncertainty is None
+        assert spectra[1].meta.get("label") == "normalised spectra"
+        assert spectra[1].meta.get("header") is not None
+
+    # No sky
+    if nspec == 3:
+        assert spectra[2].spectral_axis.unit == u.Angstrom
+        assert spectra[2].flux.unit == u.count
+        assert isinstance(spectra[2].uncertainty, StdDevUncertainty)
+        assert spectra[2].meta.get("label") is not None
+        assert spectra[2].meta.get("header") is not None
+    else:
+        assert spectra[1].spectral_axis.unit == u.Angstrom
+        assert spectra[1].flux.unit == u.count
+        assert isinstance(spectra[1].uncertainty, StdDevUncertainty)
+        assert spectra[1].meta.get("label") is not None
+        assert spectra[1].meta.get("header") is not None
+
+
+# We cannot use remote_access directly in the MIRI MRS tests, because
+# the test functions are called once for every file in the remote access
+# list, but we need to feed the entire set to the functions at once. We
+# store the file names in a list and use a dummy test method to load the
+# list via the remote_access machinery.
+#
+# MIRI MRS 1D data sets are comprised of 12 files.
+# We add one bad file to test the skip/warn on missing file functionality
+filename_list = ["bad_file.fits"]
+@remote_access([
+    {'id': '5082863', 'filename': 'combine_dithers_all_exposures_ch1-long_x1d.fits'},
+    {'id': '5082863', 'filename': 'combine_dithers_all_exposures_ch1-medium_x1d.fits'},
+    {'id': '5082863', 'filename': 'combine_dithers_all_exposures_ch1-short_x1d.fits'},
+    {'id': '5082863', 'filename': 'combine_dithers_all_exposures_ch2-long_x1d.fits'},
+    {'id': '5082863', 'filename': 'combine_dithers_all_exposures_ch2-medium_x1d.fits'},
+    {'id': '5082863', 'filename': 'combine_dithers_all_exposures_ch2-short_x1d.fits'},
+    {'id': '5082863', 'filename': 'combine_dithers_all_exposures_ch3-long_x1d.fits'},
+    {'id': '5082863', 'filename': 'combine_dithers_all_exposures_ch3-medium_x1d.fits'},
+    {'id': '5082863', 'filename': 'combine_dithers_all_exposures_ch3-short_x1d.fits'},
+    {'id': '5082863', 'filename': 'combine_dithers_all_exposures_ch4-long_x1d.fits'},
+    {'id': '5082863', 'filename': 'combine_dithers_all_exposures_ch4-medium_x1d.fits'},
+    {'id': '5082863', 'filename': 'combine_dithers_all_exposures_ch4-short_x1d.fits'},
+])
+def test_loaddata_miri_mrs(remote_data_path):
+    filename_list.append(remote_data_path)
+
+
+# loading from a list of file names
+@pytest.mark.remote_data
+def test_spectrum_list_names_miri_mrs(caplog):
+
+    # Format is explicitly set
+    with pytest.raises(FileNotFoundError):
+        specs = SpectrumList.read(filename_list, format="JWST x1d MIRI MRS")
+
+    # Skip missing file silently
+    specs = SpectrumList.read(filename_list, format="JWST x1d MIRI MRS", missing='silent')
+
+    assert len(specs) == 12
+    for spec in specs:
+        assert isinstance(spec, Spectrum1D)
+        assert spec.spectral_axis.unit == u.micron
+
+    # Warn about missing file
+    specs = SpectrumList.read(filename_list, format="JWST x1d MIRI MRS", missing='warn')
+
+    assert "Failed to load bad_file.fits: FileNotFoundError" in caplog.text
+
+    assert len(specs) == 12
+    for spec in specs:
+        assert isinstance(spec, Spectrum1D)
+        assert spec.spectral_axis.unit == u.Unit("um")
+
+    # Auto-detect format
+    specs = SpectrumList.read(filename_list[1:])
+
+    assert len(specs) == 12
+    for spec in specs:
+        assert isinstance(spec, Spectrum1D)
+        assert spec.spectral_axis.unit == u.micron
+
+
+# loading from a directory via glob
+@pytest.mark.remote_data
+def test_spectrum_list_directory_miri_mrs(tmpdir):
+
+    # copy files to temp dir. We cannot use the directory generated by
+    # remote_access, because it may have variable structure from run to
+    # run. And also because temp directories created in previous runs
+    # may still be hanging around. This precludes the use of commonpath()
+    tmp_dir = tmpdir.strpath
+    for file_path in filename_list[1:]:
+        shutil.copy(file_path, tmp_dir)
+
+    specs = SpectrumList.read(tmp_dir)
+
+    assert len(specs) == 12
+    for spec in specs:
+        assert isinstance(spec, Spectrum1D)
+        assert spec.spectral_axis.unit == u.micron
+
+
+# x1d and c1d assorted files
+
+@remote_access([
+    {'id': "5394931", 'filename':"jw00623-c1012_t002_miri_p750l_x1d.fits"},                  # pipeline 1.2.3
+    {'id': "5394931", 'filename':"jw00787-o014_s00002_niriss_f150w-gr150c-gr150r_c1d.fits"}, # pipeline 1.2.3
+    {'id': "5394931", 'filename':"jw00623-o057_t008_miri_ch1-long_x1d.fits"},                # pipeline 1.3.1
+    {'id': "5394931", 'filename':"jw00626-o064_t007_nirspec_g235h-f170lp_x1d.fits"},         # pipeline 1.3.1
+])
+def test_jwst_x1d_c1d(remote_data_path):
+
+    data = Spectrum1D.read(remote_data_path)
+
+    assert isinstance(data, Spectrum1D)
+    assert data.shape in [(388,), (5,), (1091,), (3843,)]
+    assert data.unit == u.Jy
+    assert data.spectral_axis.unit == u.um
+
+
+# utility functions to be used with list comprehension in SpectrumList checking
+def assert_multi_isinstance(a, b):
+    assert isinstance(a, b)
+
+def assert_multi_equals(a, b):
+    assert a == b
+
+
+@remote_access([
+    {'id': "5394931", 'filename':"jw00624-o027_s00001_nircam_f356w-grismr_x1d.fits"}, # pipeline 1.2.3
+])
+def test_jwst_nircam_x1d_multi_v1_2_3(remote_data_path):
+
+    data = SpectrumList.read(remote_data_path)
+
+    assert isinstance(data, SpectrumList)
+    assert len(data) == 3
+    [assert_multi_isinstance(d, Spectrum1D) for d in data]
+    [assert_multi_equals(d.shape, r) for d,r in zip(data, [(459,), (336,), (962,)])]
+    [assert_multi_equals(d.unit, u.Jy) for d in data]
+    [assert_multi_equals(d.spectral_axis.unit, u.um) for d in data]
+
+
+@remote_access([
+    {'id': "5394931", 'filename':"jw00660-o016_s00002_nircam_f444w-grismr_x1d.fits"}, # pipeline 1.3.1
+])
+def test_jwst_nircam_x1d_multi_v1_3_1(remote_data_path):
+
+    data = SpectrumList.read(remote_data_path)
+
+    assert isinstance(data, SpectrumList)
+    assert len(data) == 4
+    [assert_multi_isinstance(d, Spectrum1D) for d in data]
+    [assert_multi_equals(d.shape, r) for d,r in zip(data, [(1166,), (786,), (1157,), (795,)])]
+    [assert_multi_equals(d.unit, u.Jy) for d in data]
+    [assert_multi_equals(d.spectral_axis.unit, u.um) for d in data]
+
+
+@remote_access([
+    {'id': "5394931", 'filename':"jw00776-o003_s00083_nircam_f322w2-grismr_c1d.fits"}, # pipeline 1.2.3
+])
+def test_jwst_nircam_c1d_v1_2_3(remote_data_path):
+
+    data = SpectrumList.read(remote_data_path)
+
+    assert isinstance(data, SpectrumList)
+    assert len(data) == 2
+    [assert_multi_isinstance(d, Spectrum1D) for d in data]
+    [assert_multi_equals(d.shape, r) for d,r in zip(data, [(133,), (1139,)])]
+    [assert_multi_equals(d.unit, u.MJy/u.sr) for d in data]
+    [assert_multi_equals(d.spectral_axis.unit, u.um) for d in data]
+
+
+@remote_access([
+    {'id': "5394931", 'filename':"jw00625-o018_s00001_niriss_f090w-gr150c_c1d.fits"}, # pipeline 1.2.3
+])
+def test_jwst_niriss_c1d_v1_2_3(remote_data_path):
+
+    data = SpectrumList.read(remote_data_path)
+
+    assert isinstance(data, SpectrumList)
+    assert len(data) == 2
+    [assert_multi_isinstance(d, Spectrum1D) for d in data]
+    [assert_multi_equals(d.shape, r) for d,r in zip(data, [(56,), (107,)])]
+    [assert_multi_equals(d.unit, u.Jy) for d in data]
+    [assert_multi_equals(d.spectral_axis.unit, u.um) for d in data]
